;JT9_AD9850_Loc.asm    G4JNT  July 2014 (Ident stored at end)

;Sends Location Info as JT9 Message, generated in real time 
;Includes AD9850 Synth control, 
;Debug Info Sent on serial interface at 19200 baud for checking
;GetGPSData replaced with GetRMC, taken from NMEADisplay.  Uses Stack in A0 addressed indirectly
;  All CW stuff removed
;Derived from LMX2541 version
;Latest version calculates DDS registers instead of using pre-stored values
;FIRSTPPS added to allow for an annoying Maestro issue


    LIST P = 16F628A

		INCLUDE "P16F628A.INC"
	__config    0x3F61      ;LV Programming off , OSC XT Mode
	__idlocs    0xFAAA

	errorlevel 1, -207  ;Suppress msgs. and warnings about labels after col 1.
 

	#define     FData     	PORTA, 0        ;DDS Data on port A
	#define     WClk     	PORTA ,1	;These connections for consistency
	#define     FQud     	PORTA, 2	;NOT THE SAME as DDSPC
	#define     TxEnable    PORTA, 3

	#define     PPS         PORTB, 0
    	#define     NMEA        PORTB, 1
	#define     GPSOUT	PORTB, 2	
 	#define     TESTLINK    PORTB, 3
	#define     LEDGrn     	PORTB, 4
	#define     LEDRed      PORTB, 5
	#define     TXD      	PORTB, 6	;Debug output

	#define	    BitStore		Flags, 0
	#define     GPSValid   		Flags, 2
	#define     TestModeFlag	Flags, 1
	#define     NEGLAT      	Flags, 3
	#define     NEGLONG     	Flags, 4
	#define	    IDENTFLAG		Flags, 6
	#define     AllowSendFlag	Flags, 7

	constant	NEGPPS		=	1	;Inverted PPS (-Ve edge active)
	constant 	FIRSTPPS	=	1	;Look for first PPS pulse to indicate GPS is working

	constant    	PllFlag		=	0		;Used with AD9851

	constant	NBITS		=	d'206'	;
	constant	OPSYMBS		=	d'85'
	constant	SYMBOLDATA	=	0xA0	;Storage for Final data + sync stream
	constant	TARGETDATA	=  	SYMBOLDATA + NBITS / 4 + 1	;Storage for Interleaved  data
	constant	GPSBAUDCONST =  d'50'   ;HalfBitDelay at 4800 baud NMEA   = 5.N + 5 clocks  Value for 2.5MHz clock
	constant	TXDBAUDCONST =  d'23'	;Full bit delay for 19200 baud RS232 test port
	constant	NMEAPol		=	1	;1 = RS232 Levels, 0 = TTL


	constant	STARTSECONDS	=	1

	constant	BAUD9600	=	d'40'	;6.N + 18 = Fc / Baud   or   N ~ (Fxtal / Baud)/24 - 3
	constant	BAUD4800	=	d'83'		
	constant	JT9CONST	=	d'1406'	 ;Interrupt frequency / 1.736111

	constant	STACK0		=	0xA0		;Accessed using FSR also used for Symbol data

	constant	NBASE		=	d'25'	;Modulo for dividing down Deg*600k + Mins * 10k
	constant	NEGOFFSET	=	d'12'	;Added to MSChar as neg number flag

	constant	poly1hi	= 	0xF2D0		;JT9 (and WSPR)  convolutional encoding polynomials
	constant	poly1lo	=	0x5351
	constant	poly2hi	=	0xE461
	constant	poly2lo	=	0x3C47

	

 org 0x2100
ToneZeroFreq                            
	de	0x16, 0x8E, 0xEA, 0x10 	  	;7.0495MHz at Fc = 80MHz + 7ppm
;	de	0x0B, 0x84, 0xC6, 0x1E		;3.5996MHz



JT9Increment
	de  0x00, 0x5D, 0x34, 0xEE	;JT9A Increment * 65536	at fc = 80MHz

	
CWMsg           de      "G4JNT",0

ControlPLL      de      0			;;0 for no PLL


IdentData
	de "G4JNT LOCBCN."	;last char is overwritten by ? if GPS is invalid

;------------------------------------
 org 0
	goto	StartUp
;============================================
 org 4
ints                            ;/4 prescale &  /256 for ovflow int  = Interrupt Rate
	bcf     INTCON , T0IF   ;
	movwf   Wreg            ;Save W
	swapf   STATUS , W      ; nb.  movwf does not affect any STATUS bits
	movwf   SaveSts         ;Stave STATUS, without affecting it

	incf    IntCountLo
	btfsc   STATUS, Z
	incf    IntCountHi

        movlw   HIGH(JT9CONST)    ;Int rate/JT9CONST = symbol rate
	subwf   IntCountHi, W
	btfss   STATUS, Z
	goto    OutInterrupt

	movlw   LOW(JT9CONST)
	subwf   IntCountLo, W
	btfss   STATUS, Z
	goto    OutInterrupt
	   ;............             Here at symbol rate
	clrf    IntCountLo
	clrf    IntCountHi

	btfsc   SymbCount, 0    ;Flash Red LED at half symbol rate
	bsf     LEDRed
	btfss   SymbCount, W
	bcf     LEDRed

	movf    SymbCount       ;Test for First symbol, in which case
	btfsc   STATUS, Z       ;  don't do any DDS update
	goto    NotFirstPass
NotFirstPass

	movlw   OPSYMBS
	subwf   SymbCount, W    ;So do the testing for a full message here, 
	btfss   STATUS, Z
	goto    NotLastSymbol

	bcf     INTCON, T0IE    ;Inhibit interrupt after final symbol
	goto    OutInterrupt    ;  and don't do anything else
	;...............
NotLastSymbol
	rrf     SymbCount, W
	andlw   0x3F            ;W = SymbCount\2. Posn. of this data in EE
        addlw   TARGETDATA	;   each byte is called twice, for each pair
	movwf	FSR
        movf	INDF, W       	;JT9Data contains the two items
        movwf   JT9Data        	;  and they also remain in w

	btfss	SymbCount, 0
	swapf	JT9Data, W	;If count is even, use MS nibble
	andlw	0x0F		;Mask off the actual value

	movwf	FreqCache
	bsf	AllowSendFlag

	incf    SymbCount       ;Increment here, but tested at start allows 
                                ; the freq update at last+1 interval, ie. 207
OutInterrupt
	swapf   SaveSts , W     ;Restore STATUS
	movwf   STATUS
	swapf   Wreg            ;Restore W
	swapf   Wreg , W        ;Can't use movf as it  affects Z bit

	retfie    

;=====================================
StartUp
	clrf	INTCON
	clrf    PORTA ;Special setup to disable comparator on 'F628
	movlw   7
	movwf   CMCON

	bsf	STATUS,RP0  ;ram page 1
	movlw   b'11100000' ;
	movwf   PORTA       ;set all port A0 as outputs
	movlw   b'10001011' ;B0/1 PPS/NMEA, B2 GPS Out B3 TestLink, B4/5LEDs  B6 Debug Out
	movwf   PORTB       ;
	movlw  b'00000001'     ;Internal input, Prescalar /4  pull-ups
	movwf	OPTION_REG
	bcf     STATUS,RP0  ;ram page 0

    clrf    PORTB
    clrf    PORTA	 

    call    LongDelay		;Allow GPS Supply to stabilise
    bsf     LEDGrn

    call    SetUpGPS		
    call    LongDelay		;Allow time for reference to establish
    bsf     LEDRed
    bcf     LEDGrn
    call    LongDelay
    bcf     LEDRed
    bsf     LEDGrn
    call    LongDelay
    bcf     LEDGrn

    movlw   0
    call    SendFreq	;Start up freq
    call    LongDelay	
    movlw   0
    call    SendFreq	;Do it twice

    clrf    TMR0
    clrf    IntCountHi
    clrf    IntCountLo
    clrf    Flags
    clrf    SymbCount

 if FIRSTPPS ==1
WaitFirstPPS
   if NEGPPS == 1		;Look for first active PPS
	btfss	PPS
   else
	btfsc	PPS
   endif
	goto	PPSFound

	bsf	LEDRed		;Fast flash red lLED until GPS is first locked
	movlw	d'15'
	call	VarDelay
	bcf	LEDRed
	movlw	d'15'
	call	VarDelay
	goto	WaitFirstPPS
 endif


PPSFound
    movlw   b'10000000'     		;Enable global int, not timer overflow int yet
    movwf   INTCON


WaitGPSFirstLock            ;
    call    GetRMC
    btfss   GPSValid
    goto    WaitGPSFirstLock

    bcf     LEDRed
    bsf     LEDGrn
;...........................
MainLoop
	movlw	"."
	call	Send232
	call    GetRMC  

	movf    Seconds, W	;Seconds and Minutes are preserved
	sublw   STARTSECONDS		;and are in BCD format
	btfss   STATUS, Z	
	goto    MainLoop 
		                ;Now at minute : Startseconds
	call	CrLf		;Teminate the dots

	movlw	STACK0 - 1	;Echo back full GPRMC data
	movwf	FSR
ShowGPRMCLoop
	incf	FSR
	movf	INDF, W
	call	Send232
	movlw	0x0D
	subwf	INDF, W
	btfss	STATUS, Z
	goto	ShowGPRMCLoop	
	call	CrLf

	movf	Minutes, W
	andlw	0x0F		;Mask to units of BCD minutes
	btfsc	STATUS, Z
	bsf	IDENTFLAG	;Set Ident Flag every 10 minutes


	btfss	GPSValid
	call	DoEEData	;GPS Invalid sends ident plus marker flag in last position
				;Clears ident flag if it has been set
	movlw	"?"
	btfss	GPSValid
	movwf	SrcC
 
	btfsc	IDENTFLAG
	call	DoEEData	;Uses Fixed info stored in EE as source data

	call    SendSource
	call	GenerateJT9	;This reuses registers, so time and hash Info is lost. 

		;Start immediately  (add wait PPS later perhaps)
	bsf	TxEnable	
	clrf    SymbCount    ;Re-Initialise registers
	clrf    IntCountLo
	clrf    IntCountHi
	movlw	d'0'
	movwf	LastFreq		;Sets Tone 0 
	bcf	LEDGrn

	movlw	d'40'		;Units of 4ms.  Sequence shouldn't start until :01 second.  NMEA + Processing 
	call	VarDelay	;  + RS232 delays contribute some extra time, but still not enough on their own
	bsf     INTCON, T0IE    	;Enable the interrupt, starts JT65 sequence
JT9Loop
	btfss   AllowSendFlag
	goto    NoSend
	movf    FreqCache, W
	call    SendFreq		;Has to be done outside interrupt routine
	bcf	AllowSendFlag		; as an interrupt overrun would occur
NoSend
	btfsc   INTCON, T0IE ;Sit in here waiting for completion   
	goto    JT9Loop

	movlw   0
	call    SendFreq

	call    LongDelay
	call	LongDelay		;Give a few seconds of sync tone to clear as the decoder seems to get upset
	bcf	TxEnable
	goto	MainLoop
;----------------------------------------------------------
DoEEData		

	movlw	Src0
	movwf	FSR
	movlw	d'13'
	movwf	Counter
DoEELoop
	movf	Counter, W
	sublw	d'13'		;W = 13 - Counter, so counts from 0 to 12	
	addlw	LOW(IdentData)	;EE where 13 characters are stored	
	call	GetEE		
	movwf	INDF		;Store in Src0 - SrcC
	incf	FSR
	decfsz	Counter		;Final position is overwritten with ? if GPS is invalid
	goto	DoEELoop

	bcf	IDENTFLAG	

	return
;---------------------------------------
GenerateJT9
;  Code the message ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	movlw	Src0
	movwf	FSR

	movf	INDF, W
	call	CodeJT9Alphabet	
	movwf	A0
	clrf	A1
	clrf	A2
	clrf	A3
	movlw	d'42'		
	movwf	B0
	clrf	B1
	clrf	B2
	clrf	B3
	call	Multiply32	;Proper multiply to start with, will also flush Acc0-3


	incf	FSR
	movf	INDF, W
	call	CodeJT9Alphabet
	addwf	Acc0
	btfsc	STATUS, C
	incf	Acc1
	btfsc	STATUS, Z
	incf	Acc2
	btfsc	STATUS, Z
	incf	Acc3
	movlw	d'42'		
	movwf	B0
	call	CascadeMultiply32x8

	incf	FSR
	movf	INDF, W

	call	CodeJT9Alphabet	
	addwf	Acc0
	btfsc	STATUS, C
	incf	Acc1
	btfsc	STATUS, Z
	incf	Acc2
	btfsc	STATUS, Z
	incf	Acc3
	movlw	d'42'	
	movwf	B0
 	call	CascadeMultiply32x8

	incf	FSR
	movf	INDF, W

	call	CodeJT9Alphabet	
	addwf	Acc0
	btfsc	STATUS, C
	incf	Acc1
	btfsc	STATUS, Z
	incf	Acc2
	btfsc	STATUS, Z
	incf	Acc3
	movlw	d'42'	
	movwf	B0
 	call	CascadeMultiply32x8

	incf	FSR
	movf	INDF, W

	call	CodeJT9Alphabet	
	addwf	Acc0
	btfsc	STATUS, C
	incf	Acc1
	btfsc	STATUS, Z
	incf	Acc2
	btfsc	STATUS, Z
	incf	Acc3
	movlw	d'42'			;Gives a 27 bit result, right aligned

	movf	Acc3, W
	movwf	S0
	movf	Acc2, W
	movwf	S1
	movf	Acc1, W
	movwf	S2
	movf	Acc0, W
	movwf	S3			;First 5 characters done, but not yet aligned
		;~~~~~~~~~~~~~~~~~~~~~

 
	incf	FSR
	movf	INDF, W

	call	CodeJT9Alphabet	
	movwf	A0
	clrf	A1
	clrf	A2
	clrf	A3
	movlw	d'42'		
	movwf	B0
	clrf	B1
	clrf	B2
	clrf	B3
	call	Multiply32	;Proper multiply to start with, will also flush Acc0-3

	incf	FSR
	movf	INDF, W

	call	CodeJT9Alphabet
	addwf	Acc0
	btfsc	STATUS, C
	incf	Acc1
	btfsc	STATUS, Z
	incf	Acc2
	btfsc	STATUS, Z
	incf	Acc3
	movlw	d'42'		
	movwf	B0
	call	CascadeMultiply32x8

	incf	FSR
	movf	INDF, W

	call	CodeJT9Alphabet	
	addwf	Acc0
	btfsc	STATUS, C
	incf	Acc1
	btfsc	STATUS, Z
	incf	Acc2
	btfsc	STATUS, Z
	incf	Acc3
	movlw	d'42'	
	movwf	B0
 	call	CascadeMultiply32x8

	incf	FSR
	movf	INDF, W

	call	CodeJT9Alphabet	
	addwf	Acc0
	btfsc	STATUS, C
	incf	Acc1
	btfsc	STATUS, Z
	incf	Acc2
	btfsc	STATUS, Z
	incf	Acc3
	movlw	d'42'	
	movwf	B0
 	call	CascadeMultiply32x8

	incf	FSR
	movf	INDF, W

	call	CodeJT9Alphabet	
	addwf	Acc0
	btfsc	STATUS, C
	incf	Acc1
	btfsc	STATUS, Z
	incf	Acc2
	btfsc	STATUS, Z
	incf	Acc3
	movlw	d'42'			;Gives a 27 bit result, right aligned

	movf	Acc3, W
	movwf	S4
	movf	Acc2, W
	movwf	S5
	movf	Acc1, W
	movwf	S6
	movf	Acc0, W
	movwf	S7			;10 characters done, but not yet aligned
		;~~~~~~~~~~~~~~~~~~~~~


	incf	FSR
	movf	INDF, W

	call	CodeJT9Alphabet	
	movwf	A0
	clrf	A1
	clrf	A2
	clrf	A3
	movlw	d'42'		
	movwf	B0
	clrf	B1
	clrf	B2
	clrf	B3
	call	Multiply32	;Proper multiply to start with, will also flush Acc0-3

	incf	FSR
	movf	INDF, W

	call	CodeJT9Alphabet
	addwf	Acc0
	btfsc	STATUS, C
	incf	Acc1
	btfsc	STATUS, Z
	incf	Acc2
	btfsc	STATUS, Z
	incf	Acc3
	movlw	d'42'		
	movwf	B0
	call	CascadeMultiply32x8

	incf	FSR
	movf	INDF, W

	call	CodeJT9Alphabet	
	addwf	Acc0
	btfsc	STATUS, C
	incf	Acc1
	btfsc	STATUS, Z
	incf	Acc2
	btfsc	STATUS, Z
	incf	Acc3	

	movf	Acc0, W		;Transfer to Q registers for further processing 
	movwf	Q0
	movf	Acc1, W
	movwf	Q1
	movf	Acc2, W
	movwf	Q2
	movf	Acc3, W
	movwf	Q3
				;Last stage give a 17 bit result, leave in Q for now
				;R Aligned S0-4,  R Aligned S5-SA,  R Aligned Q2/1/0
				;      S0   S1 S2 S3      S4    S5 S6 S7        Q2       Q1    Q0
				;  00000xxx N1 N1 N1   00000yyy N2 N2 N2    0000000p Qzzzzzzz  N3

 	movlw	4
	movwf	Counter
AlignLoop1
	bcf	STATUS, C
	rlf	S7
	rlf	S6
	rlf	S5
	rlf	S4		
	decfsz	Counter		
	goto	AlignLoop1
	
	bcf	S4, 7		;Insert N3[15]  'q'
	btfsc	Q1,7		;    S4    S5 S6  S7 
	bsf	S4, 7		; qyyyyyyy N2 N2 yyyy0000  


	movlw	5
	movwf	Counter
AlignLoop2
	bcf	STATUS, C
	rlf	S7
	rlf	S6
	rlf	S5
	rlf	S4
	rlf	S3
	rlf	S2
	rlf	S1
	rlf	S0		;
	decfsz	Counter		;
 	goto	AlignLoop2

	bcf	S6, 0		;Insert N3[16]  'p'
	btfsc	Q2,0		; S0 S1 S2   S3       S4 S5  S6        S7        
	bsf	S6, 0		; N1 N1 N1 xxxqyyyy   N2 N2 yyyyyyyp  00000000	

	movf	Q1, W
	movwf	S7
	bsf	S7, 7		;Sets plain text flag
	movf	Q0, W		;  S7      S8
	movwf	S8		;1zzzzzzz  N3


	
 	clrf	A0
	clrf	A1
	clrf	A2
	clrf	A3
	
	call	CrLf	
	call	SendPacked	;################################################



;   Convolve the data

	movlw	NBITS / 2 + 1	;	The +1 ensures the resulting data is left aligned
	movwf	Counter		;

ConvolveLoop			;Shift data into RHS of A0-3
	bcf	STATUS, C
	rlf	S8
	rlf	S7
	rlf	S6
	rlf	S5
	rlf	S4
	rlf	S3
	rlf	S2
	rlf	S1
	rlf	S0
	rlf	A3
	rlf	A2
	rlf	A1
	rlf	A0

	movf	A0, W
	andlw	(poly1hi >> 8) & 0xFF
	movwf	Temp
	
	movf	A1, W
	andlw	poly1hi & 0xFF
	xorwf	Temp

	movf	A2, W
	andlw	(poly1lo >> 8) & 0xFF
	xorwf	Temp
	
	movf	A3, W
	andlw	poly1lo & 0xFF
	xorwf	Temp, W
	call	Parity		;return with  'Even' = 0 or 1
	rrf	Even, W		;Place parity bit into Carry
	call	ShiftQ		;And shift into output register
 	
	movf	A0, W
	andlw	(poly2hi >> 8) & 0xFF
	movwf	Temp
	
	movf	A1, W
	andlw	poly2hi & 0xFF
	xorwf	Temp

	movf	A2, W
	andlw	(poly2lo >> 8) & 0xFF
	xorwf	Temp
	
	movf	A3, W
	andlw	poly2lo & 0xFF
	xorwf	Temp, W
	call	Parity	
	rrf	Even,W		
	call	ShiftQ

	decfsz	Counter
	goto	ConvolveLoop	;Q0 - Q14 contains NBITS bits left justified

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~   Appear to be Ok up to here
  ;Now do the interleaving, using indirect addressing

	movlw	0xFF		;Preload so the first increment will give zero
	movwf	Ireg
	clrf	IntlvPointer
	movlw	HIGH(BitMask)
	movwf	PCLATH

IntlvLoop
	incf	Ireg		;goes from 0 - 255 irrespective of the test below
	call	BitRev		;Gets result of bit reversing Ireg into Jreg

	movlw	NBITS
	subwf	Jreg, W	
	btfsc	STATUS, C	;W = Jreg - NBITS.  If +Ve/0, (C=1) generated value is ignored
	goto	IntlvLoop

			;If valid, take the Bit pointed at by IntlvPointer (0 - NBITS-1) and
			;  put it into the bit pointed at by Jreg  

	rrf	IntlvPointer, W	; range 0 - NBITS-1
	movwf	Temp
	rrf	Temp
	rrf	Temp, W
	andlw	0x1F		;Divide by 8 to point to source byte
	addlw	Q0		;Q0, 1  etc must be stored sequentially	
	movwf	FSR		; = IntlvPointer \ 8

	movf	IntlvPointer, W
	andlw	0x07		;Get bit position 0-7, but remember that LHS = 0
	call	BitMask		;Returns with 1000000, 0100000 etc in w

	andwf	INDF, W		;Result is zero if bit is 0, something else if bit is 1
	bcf	BitStore
	btfss	STATUS, Z
	bsf	BitStore	;Source bit at location IntlvPointer has been stored 

  
  ;  Now generate the target bit position

;	incf	Jreg		;Move 1 bit to right for initial reference tone left in to show diff from JT4 up to this point

	rrf	Jreg, W	
	movwf	Temp
	rrf	Temp
	rrf	Temp, W
	andlw	0x1F
	addlw	TARGETDATA	
	movwf	FSR		;POint to byte containing destination bit

	movf	Jreg, W
	andlw	0x07		;Extract bit position
	call	BitMask
	
	btfss	BitStore
	goto	BitWasZero
	iorwf	INDF		;Force the desitination bit to 1
	goto	ThisBitDone
;..........
BitWasZero
	xorlw	0xFF	
	andwf	INDF		;Force it to 0

ThisBitDone

	incf	IntlvPointer
	movlw	NBITS
	subwf	IntlvPointer, W
	btfss	STATUS, C	;W = IntlvPointer - NBITS, if -ve (C=0) continue looping
	goto	IntlvLoop

   ;Force the redundant two bits (207 and 208)left in the final byte to zero.
	movlw	TARGETDATA + NBITS/8	;Point to the last byte
	movwf	FSR
	bcf	INDF, 0
	bcf	INDF, 1
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;#######  
		;Now have 206 bits stored in TARGETDATA array
		;   Merge into three bit symbols stored in SYMBOLDATA array and insert sync symbols into the correct place

	clrf	Counter	
GenSymbolLoop
	clrf	Temp
	call	ShiftTargetLeft
	rlf	Temp
	call	ShiftTargetLeft
	rlf	Temp
	call	ShiftTargetLeft
	rlf	Temp		;Temp contains	raw symbol value 0 - 7
				;Do the Gray Coding
	bcf	STATUS, C
	rrf	Temp, W
	xorwf	Temp		;Gray coded result  in Temp

	bcf	STATUS, C
	rrf	Counter, W	;Counter/2 since two symbols stored in each byte
	addlw	SYMBOLDATA	
	movwf	FSR		;Point to the byte to place these TWO symbols

	btfss	Counter, 0	;Start with a fresh value for first update
	clrf	INDF		;Start with a fresh byte, to allow IOR for each nibble	
	swapf	Temp, W		;High nibble contains earliest data
	btfsc	Counter, 0	;Test for high/low nibble to store this symbol	
	movf	Temp, W		;If Counter is odd, the low nibble is stored
	iorwf	INDF		;Store each one of the pair 

DoneByteStore
	incf	Counter
	movlw	NBITS/3 + 1	;Number of final symbols (+1 to capture MS nibble of last one)
	subwf	Counter, W	;W = Counter2 - Target, if +ve/zero, C=1, stop and exit
	btfss	STATUS, C
	goto	GenSymbolLoop	

;#######

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
			;Now have set of 69 data symbols of value 1 - 8 stored in SYMBOLDATA array
			;   Mix with SYnc vector

	clrf	Counter		;Counts output symbols   up to OPSYMBS
	clrf	Counter2	;Counts source symbols, count limited by number of zeros in sync vector table
OpLoop
	rrf	Counter, w	;Sync/data flags stored 8 per byte
	movwf	Temp
	rrf	Temp
	rrf	Temp, W
	andlw	0x0F		;Counter/8 = Byte containing this set of 8 output states	
	call	SyncTable	;8 possible sync/data flag bits
	movwf	Temp		

	movf	Counter, W	;Get bit number within this byte
	andlw	0x07
	call	BitMask		;Returns with one bit set in appropriate position (MS bit first)
	andwf	Temp		;Result is zero (Z=1) if this is a data symbol, any other value if sync

	movlw	0		;(movlw does not affect the Z bit)
	btfsS	STATUS, Z	;none-zero if a sync symbol
	goto	DoSync		;Enters that with W = 0 for sync symbol value

	bcf	STATUS, C
	rrf	Counter2, W	;It's a data symbol, so point to the byte
	addlw	SYMBOLDATA	; containing next (pair of) input symbols
	movwf	FSR

	swapf	INDF, W		;Select appropriate nibble
	btfsc	Counter2, 0	;Hi nibble comes first (even count)
	movf	INDF, W
	andlw	0x07		;Result in W
	addlw	1		;Data symbols 1 - 8
	incf	Counter2	;Update the input symbol counter

DoSync				;Jump in here with W=0 if sync, or keep 1-8 data symbol value
	movwf	Temp		;Store this symbol value

	bcf	STATUS, C	;
	rrf	Counter, W	;Now identify the target byte for the pair
	addlw	TARGETDATA	
	movwf	FSR		;Point to the byte to place these TWO symbols

	btfss	Counter, 0	;Start with a fresh value for first update
	clrf	INDF		;Start with a fresh byte, to allow IOR for each nibble	

	swapf	Temp, W		;High nibble contains earliest data
	btfsc	Counter, 0	;Test for high/low nibble to store this symbol	
	movf	Temp, W		;If Counter is odd, the low nibble is stored
	iorwf	INDF		;Store each one of the pair 

	incf	Counter
	movlw	OPSYMBS		;Number of final symbols
	subwf	Counter, W	;W = Counter - Target, if +ve/zero, C=1, stop and exit
	btfss	STATUS, C
	goto	OpLoop
			;Now have an array of 85 output sysmbls stored in TARGETDATA, MS nibble first
;~~~~~~~~~~~~~~~~
	call	CrLf
	call	SendResults	;#####################################################

	return
;=============================================================================
Parity			;Returns with parity result in Even and W
	clrf	Even
	movwf	Temp
ParityLoop
	movf	Temp		
	btfsc	STATUS, Z	;Do the parity loop until N = 0
	return			;last state of Even is still in W from previous pass

	decf	Temp, W
	andwf	Temp		;Temp = Temp AND (Temp -1)
	movlw	1
	xorwf	Even
	movf	Even, W		;Put it into W in case we return next time around
	goto	ParityLoop
;-----------------------------------------------------------
BitRev
	clrf	Jreg
	btfsc	Ireg , 0
	bsf	Jreg , 7
	btfsc	Ireg , 1
	bsf	Jreg , 6
	btfsc	Ireg , 2
	bsf	Jreg , 5
	btfsc	Ireg , 3
	bsf	Jreg , 4
	btfsc	Ireg , 4
	bsf	Jreg , 3
	btfsc	Ireg , 5
	bsf	Jreg , 2
	btfsc	Ireg , 6
	bsf	Jreg , 1
	btfsc	Ireg , 7
	bsf	Jreg , 0

	return
;-----------------------------------------------------------
ShiftQ			;Puts Next data bit in Carry into RHSide
	rlf	Q19
	rlf	Q18
	rlf	Q17
	rlf	Q16
	rlf	Q15
	rlf	Q14
	rlf	Q13
	rlf	Q12
	rlf	Q11
	rlf	Q10
	rlf	QF
	rlf	QE
	rlf	QD
	rlf	QC
	rlf	QB
	rlf	QA
	rlf	Q9
	rlf	Q8
	rlf	Q7
	rlf	Q6
	rlf	Q5
	rlf	Q4
	rlf	Q3
	rlf	Q2
	rlf	Q1
	rlf	Q0

	return
;------------------------------
ShiftTargetLeft
	bsf	STATUS, RP0		;This RAM area is Page 1, normally accesed via FSR

	rlf	TARGETDATA + d'25'
	rlf	TARGETDATA + d'24'
	rlf	TARGETDATA + d'23'
	rlf	TARGETDATA + d'22'
	rlf	TARGETDATA + d'21'
	rlf	TARGETDATA + d'20'
	rlf	TARGETDATA + d'19'
	rlf	TARGETDATA + d'18'
	rlf	TARGETDATA + d'17'
	rlf	TARGETDATA + d'16'
	rlf	TARGETDATA + d'15'
	rlf	TARGETDATA + d'14'
	rlf	TARGETDATA + d'13'
	rlf	TARGETDATA + d'12'
	rlf	TARGETDATA + d'11'
	rlf	TARGETDATA + d'10'
	rlf	TARGETDATA + d'9'
	rlf	TARGETDATA + d'8'
	rlf	TARGETDATA + d'7'
	rlf	TARGETDATA + d'6'
	rlf	TARGETDATA + d'5'
	rlf	TARGETDATA + d'4'
	rlf	TARGETDATA + d'3'
	rlf	TARGETDATA + d'2'
	rlf	TARGETDATA + d'1'
	rlf	TARGETDATA 

	bcf	STATUS, RP0
	return
;------------------------------
HexByte          	;Converts Byte to two hex charaters   in BcdTemp2/1
    movwf   SendChar
    swapf   SendChar, w
    andlw   0x0F
    movwf   Temp2
    sublw   9       ;W = 9 - W  if -ve (C=0)  letter A - F
    movlw   0
    btfss   STATUS, C
    movlw   7
    addwf   Temp2, w
    addlw   0x30
    movwf   BcdTemp2

    movf    SendChar, w
    andlw   0x0F
    movwf   Temp2
    sublw   9
    movlw   0
    btfss   STATUS, C
    movlw   7
    addwf   Temp2, w
    addlw   0x30
    movwf   BcdTemp1

    return
;
;----------------------
Hex232           ;Send byte as hex on RS232 link
    call    HexByte
    movf    BcdTemp2, W
    call    Send232
    movf    BcdTemp1, W
    call    Send232
    movlw   " "
    call    Send232
    return
;--------------------------------------
Send232            
	movwf   Temp2
 if NMEAPol == 1
	bsf     TXD
 else
	bcf     TXD         ;start bit
 endif
	call    BitDelay    ;
	movlw   8
	movwf   Counter2
SendDatLoop
	rrf     Temp2        ;bit into C


 if NMEAPol == 1
	btfsc   STATUS , C
	bcf     TXD
	btfss   STATUS , C
	bsf     TXD         ;could have 2us jitter depending on 1/0
 else
	btfsc   STATUS , C
	bsf     TXD
	btfss   STATUS , C
	bcf     TXD         ;could have 2us jitter depending on 1/0
 endif
	call    BitDelay
	decfsz  Counter2
	goto    SendDatLoop     ;loop 8 + BitDelay long

 if NMEAPol == 1
	bcf     TXD
 else
	bsf     TXD
 endif
	call    BitDelay    
	call    BitDelay    ;Two stop bits

	return
;-----------------------------------------
BitDelay     
	movlw   TXDBAUDCONST   
	movwf   DelCount
BaudDelLoop		
	nop
	nop
	decfsz  DelCount
	goto    BaudDelLoop
	nop
	nop
	return
;----------------------
GetEE         ;different from ;F84   EE registers all in bank 1 now 
	bsf     STATUS,RP0      ;ram page 1
	movwf   EEADR
	bsf     EECON1 , RD
	movf    EEDATA , W
	bcf     STATUS , RP0    ;ram page 0
	return
;-----------------------
CodeJT9Alphabet		;Enter with character to encode in W
	movwf	Temp	

	movlw	"+"		;First check for specials (all less than "0")
	subwf	Temp, W
	movlw	d'37'
	btfsc	STATUS, Z
	return

	movlw	"-"
	subwf	Temp, W
	movlw	d'38'
	btfsc	STATUS, Z
	return

	movlw	"."
	subwf	Temp, W
	movlw	d'39'
	btfsc	STATUS, Z
	return

	movlw	"/"
	subwf	Temp, W
	movlw	d'40'
	btfsc	STATUS, Z
	return

	movlw	"?"
	subwf	Temp, W
	movlw	d'41'
	btfsc	STATUS, Z
	return

	movlw	"0"
	subwf	Temp, W		;W = ASCII - "0"
	btfss	STATUS, C	;if -ve (C=0) invalid, send a space
	goto	DoSpace

	movlw	"9" + 1
	subwf	Temp, W		;W = ASCII - 0x3A
	btfss	STATUS, C	;if -ve (C=0), its a number
	goto	DoNumber

	movlw	"A"
	subwf	Temp, W		;W = ASCII - "A"
	btfss	STATUS, C	;If -ve, not a letter
	goto	DoSpace

	movlw	"Z" + 1
	subwf	Temp, W
	btfss	STATUS, C
	goto	DoLetter

	movlw	"a"
	subwf	Temp, W		;W = ASCII - "A"
	btfss	STATUS, C	;If -ve, not a letter
	goto	DoSpace

	movlw	"z" + 1
	subwf	Temp, W		;W = ASCII - "z"
	btfss	STATUS, C	;If -ve (C=0), valid LC letter
	goto	DoLowerLetter



DoSpace				;All other characters fall though to here and are treated as a space
	movlw	d'36'
	return
;......
DoNumber
	movf	Temp, W
	andlw	0x0F
	return
;..........
DoLetter
	movlw	"A" - d'10'	;Letters coded 10 - 35, so subtract ("A" - d'10)
	subwf	Temp, W
	return
;..........
DoLowerLetter
	movlw	"a" - d'10'
	subwf	Temp, W
	return
;-------------------------------
CascadeMultiply32x8	;Quick solution to setting B1-3 to zero each time and moving accum result back to A
	clrf	B1
	clrf	B2
	clrf	B3
	movf	Acc3, W
	movwf	A3
	movf	Acc2, W
	movwf	A2
	movf	Acc1, W
	movwf	A1
	movf	Acc0, W
	movwf	A0

Multiply32  ;32 bit * 32 bit multiplication, more efficient method
    Clrf    Acc0        ;Input A0/1/2/3 and B0/1/2/3, output Acc0/1/2/3/4/5/6/7
    clrf    Acc1        ;A destroyed,  B kept
    clrf    Acc2        
    clrf    Acc3       
    clrf    Acc4
    clrf    Acc5
    clrf    Acc6
    clrf    Acc7

    movlw   d'32'       ;A and B 24 bit
    movwf   Counter
MultLoop            
    bcf     STATUS, C
    rrf     A3
    rrf     A2          ;Working bit into carry
    rrf     A1
    rrf     A0          ;Effectivley adds shifted version of B
    btfss   STATUS, C   ; into Acc depending on working bit of A
    goto    NoMult      ; by shifting the Accumulator instead of B itself

    movf    B0, W       ;get shifted B0 value
    addwf   Acc4        ; and add into Qumulator if 1 in 
    btfss   STATUS, C   ; appropriate bit in Avalue
    goto    MulAdd1
    incf    Acc5
    btfss   STATUS, Z
    goto    MulAdd1     
    incf    Acc6
    btfss   STATUS, Z
    goto    MulAdd1
    movlw   1           ;Need more complex addition routine for last Acc as
    addwf   Acc7        ; final Carry has to be preserved for the shift
MulAdd1
    movf    B1, W
    addwf   Acc5
    btfss   STATUS, C
    goto    MulAdd2
    incf    Acc6
    btfss   STATUS, Z
    goto    MulAdd2
    movlw   1
    addwf   Acc7

MulAdd2
    movf    B2, W
    addwf   Acc6
    btfss   STATUS, C
    goto    MulAdd3
    movlw   1
    addwf   Acc7
MulAdd3
    movf    B3, W
    addwf   Acc7

NoMult                  ;Need to rotate in carry from final addition.  C is
    rrf     Acc7        ; already zero if we've jumped in here.
    rrf     Acc6
    rrf     Acc5        
    rrf     Acc4
    rrf     Acc3
    rrf     Acc2
    rrf     Acc1
    rrf     Acc0
    decfsz  Counter
    goto    MultLoop

    return              ;Acc7/6/5/4/3/2/1/0 = A * B     
;------------------------------------------------------------------
CascadeDivide32		;Copy old Quotient to A and divide by B0 again
	movf	Acc3, W
	movwf	A3
	movf	Acc2, W
	movwf	A2
	movf	Acc1, W
	movwf	A1
	movf	Acc0, W
	movwf	A0
Divide32x8              ;[32bit] A / [8 bit] B0  = [32 bit] Quotient in Acc3/0, 
    movlw   d'32'       ; remainder [8 bit] in Acc5
    movwf   Counter
    clrf    Acc5
Divide8Loop
    bcf     STATUS, C
    rlf     A0
    rlf     A1
    rlf     A2
    rlf     A3          
    rlf     Acc5              ;Rotate A through Remainder
    movf    B0, W               
    subwf   Acc5, W           ;Subtract B from R
    btfsc   STATUS, C        ;If sucessful, (C=1) place the result in R,
    movwf   Acc5              ;  otherwise keep R as before
    rlf     Acc0              ;Use the carry bit from the subtraction
    rlf     Acc1              ; to build up Acc
    rlf     Acc2
    rlf     Acc3
    decfsz  Counter
    goto    Divide8Loop

    return			;Remainder [8 bit] in Acc5
;-------------------------
GetGPSByte	

 if NMEAPol == 1       ;1 = RS232 levels, 0 = TTL
    btfss   NMEA        ;Data inverted for RS232 polarity
 else
    btfsc   NMEA
 endif

    goto    GetGPSByte     ;Loop waiting for start bit
    call    GPSHalfBitDelay    ;Centre of start bit

  if NMEAPol == 1
    btfss   NMEA        ;Data inverted for RS232 polarity
  else
    btfsc   NMEA
  endif
    goto    GetGPSByte     ;Glitch

	movlw   8    
	movwf   Counter
    clrf    Temp        ;probably not needed, but good practice

InitGPSByteLoop
	    call    GPSHalfBitDelay
	    call    GPSHalfBitDelay    ;
	    nop
	bcf     STATUS , C      ;start in known state

  if NMEAPol == 1
    btfss   NMEA            ;data polarity is inverted for RS232 polarity
  else
    btfsc   NMEA
  endif

    bsf     STATUS , C      ;Data into Carry
    rrf     Temp            ;and then into Temp, LSB first
    decfsz  Counter
    goto    InitGPSByteLoop ;loop is 8 + 2 * HalfBitDelay long

    call    GPSHalfBitDelay  
    call    GPSHalfBitDelay  


                ;We now have the entire stop bit left to use
    movf    Temp , W    ; this data, approx 200 clocks
    return          ;Exit with received byte in W and Temp
;-----------------------------------------
GPSHalfBitDelay         
    movlw   GPSBAUDCONST 
    movwf   DelCount
HalfGPSBaudDelLoop
    nop
    nop
    decfsz  DelCount
    goto    HalfGPSBaudDelLoop

    return
;---------------------------
CrLf                    ;Uses less space than storing in string
    movlw   0x0D
    call    Send232
    movlw   0x0A
    call    Send232
    return
;--------------------------- 
GetRMC
		;$GPRMC,152914,A,5054.5854,N,00117.4020,W,000.0,000.0,230615,003.5,W*70
    bsf     LEDGrn
    btfss   GPSValid    ;Long flash if valid
    bsf     LEDRed	
    btfsc   GPSValid
    bcf     LEDRed

    movlw   STACK0
    movwf   FSR
    
NMEALoop1
    call    GetGPSByte
    movwf   INDF
    movlw   "$"
    subwf   INDF, W
    btfss   STATUS, Z
    goto    NMEALoop1       ;Keep looping until starting $ is detectd


NMEALoop2
    incf    FSR
    call    GetGPSByte
    movwf   INDF        ;Push received characters into buffer until
    movlw   0x0D        ;  a [cr] is found terminating the sentence
    subwf   INDF, W
    btfss   STATUS, Z
    goto    NMEALoop2

;    movwf   PosCR           ;Save position of [cr] for later useage

                ;Now look for GPRMC in STACK0 +1 /+5 
    movlw   STACK0 + 1      
    movwf   FSR
    movlw   "G"
    subwf   INDF, W
    btfss   STATUS, Z
    goto    GetRMC        ;


    incf    FSR
    movlw   "P"
    subwf   INDF, W
    btfss   STATUS, Z
    goto    GetRMC        ;

    incf    FSR
    movlw   "R"
    subwf   INDF, W
    btfss   STATUS, Z
    goto    GetRMC        ;

    incf    FSR
    movlw   "M"
    subwf   INDF, W
    btfss   STATUS, Z
    goto    GetRMC        ;

    incf    FSR
    movlw   "C"
    subwf   INDF, W
    btfss   STATUS, Z
    goto    GetRMC        ;

    btfss   GPSValid    ;Long flash if valid
    bcf     LEDRed
    btfsc   GPSValid
    bsf     LEDRed

    clrf    CommaCount

                ;Now step through the RMC sentence counting 
CommaLoop           ;  commas to identify each field    
    incf    FSR

    movlw   0x0D
    subwf   INDF, W
    btfsc   STATUS, Z
    return

    movlw   ","
    subwf   INDF, W
    btfss   STATUS, Z
    goto    CommaLoop

    incf    CommaCount
    movlw   1
    subwf   CommaCount, W
    btfsc   STATUS, Z
    call    DoTime

    movlw   2
    subwf   CommaCount, W
    btfsc   STATUS, Z
    call    DoStatus

    movlw   3
    subwf   CommaCount, W
    btfsc   STATUS, Z
    call    DoLat

    movlw   4
    subwf   CommaCount, W
    btfsc   STATUS, Z
    call    DoNS

    movlw   5
    subwf   CommaCount, W
    btfsc   STATUS, Z
    call    DoLong

    movlw   6
    subwf   CommaCount, W
    btfsc   STATUS, Z
    call    DoEW

    movlw   7
    subwf   CommaCount, W
    btfsc   STATUS, Z
    call    DoSog

	;8 is Cog, not used

    movlw   9
    subwf   CommaCount, W
    btfsc   STATUS, Z
    call    DoDate
        
        
    goto    CommaLoop
    
;....................................
;--------------------------------------
DoTime
    incf    FSR
    swapf   INDF, w
    andlw   0xF0
    movwf   Hours

    incf    FSR
    movf    INDF, w
    andlw   0x0F
    addwf   Hours

    incf    FSR
    swapf   INDF, w
    andlw   0xF0
    movwf   Minutes

    incf    FSR
    movf    INDF, w
    andlw   0x0F
    addwf   Minutes

    incf    FSR
    swapf   INDF, w
    andlw   0xF0
    movwf   Seconds

    incf    FSR
    movf    INDF, w
    andlw   0x0F
    addwf   Seconds

    return
;--------------------------------------
DoDate
    incf    FSR
    swapf   INDF, w
    andlw   0xF0
    movwf   Day

    incf    FSR
    movf    INDF, w
    andlw   0x0F
    addwf   Day

    incf    FSR
    swapf   INDF, w
    andlw   0xF0
    movwf   Month

    incf    FSR
    movf    INDF, w
    andlw   0x0F
    addwf   Month

    incf    FSR
    swapf   INDF, w
    andlw   0xF0
    movwf   Year

    incf    FSR
    movf    INDF, w
    andlw   0x0F
    addwf   Year

    return
;--------------------------------------
DoStatus
    incf    FSR

    movlw   "A"
    subwf   INDF, w
    btfsc   STATUS, Z
    bsf     GPSValid

    movlw   "V"
    subwf   INDF, w
    btfsc   STATUS, Z
    bcf     GPSValid

    return
;--------------------------------------
DoLong
    incf    FSR
    movf    INDF, w         ;100Deg
    andlw   0x0F
    movwf   BCDMe

    incf    FSR
    swapf   INDF, w
    andlw   0xF0
    movwf   BCDLo

    incf    FSR
    movf    INDF, w         ;Deg
    andlw   0x0F
    addwf   BCDLo
 
    clrf    BCDHi
    clrf    BCDVi
    call    BCDtoBin24
    movf    A0, w
    movwf   LongD

			;Generate 600000 * Long   in registers   Acc3/0

	movf	LongD, W
	movwf	B0
	clrf	B1
	clrf	B2
	clrf	B3

	clrf	A3
	movlw	(d'600000' >> d'16') & 0xFF
	movwf	A2
	movlw	(d'600000' >> 8) & 0xFF
	movwf	A1
	movlw	d'600000' & 0xFF
	movwf	A0

	call	Multiply32	;Result in Acc3/0



    incf    FSR
    swapf   INDF, w
    andlw   0xF0 
    movwf   BCDHi

    incf    FSR
    movf    INDF, w         ;unit '
    andlw   0x0F
    addwf   BCDHi


    incf    FSR             ;bypass decimal point
    incf    FSR
    swapf   INDF, w
    andlw   0xF0
    movwf   BCDMe

    incf    FSR
    movf    INDF, w         ; '/100
    andlw   0x0F
    addwf   BCDMe

    incf    FSR
    swapf   INDF, w
    andlw   0xF0
    movwf   BCDLo

    incf    FSR
    movf    INDF, w         ; '/10000
    andlw   0x0F
    addwf   BCDLo


    clrf    BCDVi
    call    BCDtoBin24


			;Now have LongD and 10000 * Mins in    A0/1/2/3


	movf	A0, W		;Add on to modified degrees
	addwf	Acc0

	movf	A1, W
	btfsc	STATUS, C
	incfsz	A1, W		
	addwf	Acc1

	movf	A2, W
	btfsc	STATUS, C
	incfsz	A2, W
	addwf	Acc2		;B = A + B

	movf	A3, W
	btfsc	STATUS, C
	incfsz	A3, W
	addwf	Acc3
			;Now have Deg + 10000 * Mins in    Acc0/1/2/3


	movlw	NBASE
	movwf	B0
	call	CascadeDivide32		;Divides accumulator by B0 and places value back
	movf	Acc5, W			; into Acc3/0 with remainder in Acc5
	addlw	"A"
	movwf	SrcC

	movlw	NBASE
	movwf	B0
	call	CascadeDivide32		
	movf	Acc5, W			
	addlw	"A"
	movwf	SrcB

	movlw	NBASE
	movwf	B0
	call	CascadeDivide32		
	movf	Acc5, W			
	addlw	"A"
	movwf	SrcA

	movlw	NBASE
	movwf	B0
	call	CascadeDivide32		
	movf	Acc5, W			
	addlw	"A"
	movwf	Src9

	movlw	NBASE
	movwf	B0
	call	CascadeDivide32		
	movf	Acc5, W			
	addlw	"A"
	movwf	Src8

	movf	Acc0, W			
	addlw	"A"
	btfsc	NEGLONG
	addlw	NEGOFFSET
	movwf	Src7

	movlw	"-"
	movwf	Src6


    return
;--------------------------------------
DoLat
    incf    FSR
    swapf   INDF, w
    andlw   0xF0
    movwf   BCDLo

    incf    FSR
    movf    INDF, w         ;Deg
    andlw   0x0F
    addwf   BCDLo


    clrf    BCDVi
    clrf    BCDHi
    clrf    BCDMe
    call    BCDtoBin24
    movf    A0, w
    movwf   LatD

			;Generate 600000 * Lat   in registers   Acc3/0

	movf	LatD, W
	movwf	B0
	clrf	B1
	clrf	B2
	clrf	B3

	clrf	A3
	movlw	(d'600000' >> d'16') & 0xFF
	movwf	A2
	movlw	(d'600000' >> 8) & 0xFF
	movwf	A1
	movlw	d'600000' & 0xFF
	movwf	A0

	call	Multiply32	;Result in Acc3/0

    incf    FSR
    swapf   INDF, w
    andlw   0xF0
    movwf   BCDHi

    incf    FSR
    movf    INDF, w
    andlw   0x0F
    addwf   BCDHi

    incf    FSR             ;Decimal point

    incf    FSR
    swapf   INDF, w
    andlw   0xF0
    movwf   BCDMe

    incf    FSR
    movf    INDF, w         ; '/100
    andlw   0x0F
    addwf   BCDMe


    incf    FSR
    swapf   INDF, w
    andlw   0xF0
    movwf   BCDLo

    incf    FSR
    movf    INDF, w         ; '/10000
    andlw   0x0F
    addwf   BCDLo

    clrf    BCDVi
    call    BCDtoBin24		;10k * MIns in A3/0



	movf	A0, W		;Add on to modified degrees
	addwf	Acc0

	movf	A1, W
	btfsc	STATUS, C
	incfsz	A1, W		
	addwf	Acc1

	movf	A2, W
	btfsc	STATUS, C
	incfsz	A2, W
	addwf	Acc2		;B = A + B

	movf	A3, W
	btfsc	STATUS, C
	incfsz	A3, W
	addwf	Acc3
			;Now have Deg + 10000 * Mins in    Acc0/1/2/3

	movlw	NBASE
	movwf	B0
	call	CascadeDivide32		;Divides accumulator by B0 and places value back
	movf	Acc5, W			; into Acc3/0 with remainder in Acc5
	addlw	"A"
	movwf	Src5

	movlw	NBASE
	movwf	B0
	call	CascadeDivide32		
	movf	Acc5, W			
	addlw	"A"
	movwf	Src4

	movlw	NBASE
	movwf	B0
	call	CascadeDivide32		
	movf	Acc5, W			
	addlw	"A"
	movwf	Src3

	movlw	NBASE
	movwf	B0
	call	CascadeDivide32		
	movf	Acc5, W			
	addlw	"A"
	movwf	Src2

	movlw	NBASE
	movwf	B0
	call	CascadeDivide32		
	movf	Acc5, W			
	addlw	"A"
	movwf	Src1

	movf	Acc0, W			
	addlw	"A"
	btfsc	NEGLAT
	addlw	NEGOFFSET

	movwf	Src0


    return
;--------------------------------------
DoNS
    incf    FSR
    movlw   "S"
    subwf   INDF, w
    bcf     NEGLAT
    btfsc   STATUS, Z
    bsf     NEGLAT
    return
;--------------------------------------
DoEW
    incf    FSR
    movlw   "W"
    subwf   INDF, w
    bcf     NEGLONG
    btfsc   STATUS, Z
    bsf     NEGLONG
    return
;--------------------------------------
DoSog
    movlw   "0"         ;This is a variable field length, so need to first
    movwf   Sog3        ;  find the decimal point and work from there
    movwf   Sog4

SogLoopDP               ;Find the position of the decimal point
    incf    FSR
    movlw   "."
    subwf   INDF, w
    btfss   STATUS, Z
    goto    SogLoopDP

    incf    FSR         ;Point to tenths
    movf    INDF, w
    movwf   Sog1
    decf    FSR
    decf    FSR         ;Point to units
    movf    INDF, W
    movwf   Sog2        ;Variable length field, so now step backwards
                        ;  continuously checking for initial comma.
    decf    FSR         ;Point to Tens or may be comma
    movlw   ","
    subwf   INDF, w
    btfsc   STATUS, Z
    goto    DoneSogZ

    movf    INDF, W
    movwf   Sog3        ;tens

    decf    FSR         ;Point to Hundreds or may be comma
    movlw   ","
    subwf   INDF, w
    btfsc   STATUS, Z
    goto    DoneSogZ

    movf    INDF, W
    movwf   Sog4        ;hundreds

DoneSogZ                ;Now step forward to terminating comma
    incf    FSR
    movlw   ","
    subwf   INDF, w
    btfss   STATUS, Z
    goto    DoneSogZ    

    decf    FSR         ;Exit with FSR pointing to last item in field
    return
;--------------------------------------
BCDtoBin24      ;Packed BCD in BCDVi/Hi/Me/Lo to 24 bit binary in A3/2/1/0
    clrf    A2      ;Needs BcdTemp1 and BcdTemp2
    clrf    A1      ;  may be able to reuse Acc registers in some cases

    swapf   BCDVi, W    ;get 10 000 000s digit
    andlw   0x0F
    movwf   A0     
    call    Mult10

    movf    BCDVi, W    ;get 1 000 000s digit
    andlw   0x0F
    addwf   A0          ;No possible overflow up to here, max possible 99
    call    Mult10      

    swapf   BCDHi, W    ;100 000
    andlw   0x0F
    addwf   A0
    btfsc   STATUS, C
    incf    A1
    call    Mult10

    movf    BCDHi, W    ;10 000s digit
    andlw   0x0F        ;
    addwf   A0          ;  max possible count to here is 9999
    btfsc   STATUS, C
    incf    A1
    call    Mult10

    swapf   BCDMe, W    ;1000s digit
    andlw   0x0F
    addwf   A0
    btfss   STATUS, C
    goto    No1k
    incf    A1
    btfsc   STATUS, Z
    incf    A2
No1k
    call    Mult10
 
    movf    BCDMe, W    ;100s digit
    andlw   0x0F
    addwf   A0
    btfss   STATUS, C
    goto    NoHund
    incf    A1
    btfsc   STATUS, Z
    incf    A2
NoHund
    call    Mult10

    swapf   BCDLo, W    ;10s digit
    andlw   0x0F
    addwf   A0
    btfss   STATUS, C
    goto    NoTen
    incf    A1
    btfsc   STATUS, Z
    incf    A2
NoTen
    call    Mult10

    movf    BCDLo, W    ;Units digit
    andlw   0x0F
    addwf   A0
    btfss   STATUS, C
    goto    NoUnit
    incf    A1
    btfsc   STATUS, Z
    incf    A2
NoUnit
    return
;....................
Mult10                  ;Multiplies A2/1/0 by 10, Uses BcdTemp2/1
    movf    A2, W
    movwf   BcdTemp2
    movf    A1, W
    movwf   BcdTemp1
    movf    A0, W      ;Store original values in BcdTemp2/1 and W

    bcf     STATUS, C
    rlf     A0
    rlf     A1
    rlf     A2
    bcf     STATUS, C
    rlf     A0
    rlf     A1
    rlf     A2         ;A = 4 * A

    addwf   A0
    btfss   STATUS, C
    goto    NoMult10Inc     ;This inability to double inc without
    incf    A1              ;  jumping is getting annoying !!!!!
    btfsc   STATUS, Z
    incf    A2
NoMult10Inc
    movf    BcdTemp1, W
    addwf   A1
    btfsc   STATUS, C
    incf    A2

    movf    BcdTemp2, W
    addwf   A2         ;A = A * 5

    bcf     STATUS, C   
    rlf     A0
    rlf     A1
    rlf     A2         ;A = 10 * A
    return
;-------------------------



;----------------------------------
SendSource
	movlw	Src0
	movwf	FSR

	movf	INDF, W
	call	Send232
	incf	FSR
	movf	INDF, W
	call	Send232
	incf	FSR
	movf	INDF, W
	call	Send232
	incf	FSR
	movf	INDF, W
	call	Send232
	incf	FSR
	movf	INDF, W
	call	Send232
	incf	FSR
	movf	INDF, W
	call	Send232
	incf	FSR
	movf	INDF, W
	call	Send232
	incf	FSR
	movf	INDF, W
	call	Send232
	incf	FSR
	movf	INDF, W
	call	Send232
	incf	FSR
	movf	INDF, W
	call	Send232
	incf	FSR
	movf	INDF, W
	call	Send232
	incf	FSR
	movf	INDF, W
	call	Send232
	incf	FSR
	movf	INDF, W
	call	Send232

	movlw	"*"
	call	Send232
	call	CrLf

	return
;------------------------------------------------
SendPacked

	movlw	"P"
	call	Send232
	movlw	"a"
	call	Send232
	movlw	"c"
	call	Send232
	movlw	"k"
	call	Send232
	movlw	"e"
	call	Send232
	movlw	"d"
	call	Send232
	movlw	" "
	call	Send232
	movlw	" "
	call	Send232

	movf	S0, W
	call	Hex232
	movf	S1, W
	call	Hex232	
	movf	S2, W
	call	Hex232	
	movf	S3, W
	call	Hex232	
	movf	S4, W
	call	Hex232	
	movf	S5, W
	call	Hex232	
	movf	S6, W
	call	Hex232	
	movf	S7, W
	call	Hex232	
	movf	S8, W
	call	Hex232	

	call	CrLf

	return
;----------------------------------------------
SendResults
	movlw	"S"
	call	Send232
	movlw	"y"
	call	Send232
	movlw	"m"
	call	Send232
	movlw	"b"
	call	Send232
	movlw	"o"
	call	Send232
	movlw	"l"
	call	Send232
	movlw	"s"
	call	Send232
		
	movlw	TARGETDATA
	movwf	FSR
	movlw	OPSYMBS / 2 + 1
	movwf	B0
FinalPRLoop	
	movf	B0, W
	sublw	OPSYMBS / 2 + 1		;To get an increasing count from zero
	andlw	0x07
	btfsc	STATUS, Z
	call	CrLf	;Format it into groups of 8
	
	movf	INDF, W
	call	Hex232
	incf	FSR
	decfsz	B0
	goto	FinalPRLoop

	call	CrLf
		
	return
;------------------------------------
ShowStackContents           ;Just here as a debugging aid
    movlw   STACK0
    movwf   FSR
    movlw   d'30'
    movwf   DebugTemp
QQQLOOP
    movf    INDF, w
    call    Hex232
    incf    FSR
    decfsz  DebugTemp
    goto    QQQLOOP
    call    CrLf
    return

;-------------------------------

WaitPPS                 ;In case PPS is still high
    btfsc   PPS         ; look for a low to high transition
    goto    WaitPPS

WaitRefPPS              ;Now wait for the rising edge itself
    btfss   PPS        
    goto    WaitRefPPS 

    return
;----------------------

;-----------------------
LongDelay           			;Approx 1 second delay here
	movlw   d'250'
VarDelay			
	movwf   DelCount2		;Or a variable delay in units of 4ms in W here
LongDelLoop
	call    Delay1ms
	call    Delay1ms
	call    Delay1ms
	call    Delay1ms
	decfsz  DelCount2
	goto    LongDelLoop
	return
;-------------------------
Delay1ms        ;5N+5 =  1250 clocks, * 2 = 2500 clocks = 1ms
	movlw   d'249'
DelayNms
	movwf   DelCount
Del1msLoop
	nop
	nop
	decfsz  DelCount
	goto    Del1msLoop
	movlw   d'249'
	movwf   DelCount
Del1ms2Loop
	nop
	nop
	decfsz  DelCount
	goto    Del1ms2Loop
	nop
	nop
	nop
	nop
	return
;-------------------------------
SendFreq		;Takes JTxx code 0 - nn, constructs registers and sends to synth chip
        movwf   A0              ;Input to multiplication routine

        movlw   LOW(ToneZeroFreq) + 3    ;Get base carrier frequency
	call    GetEE
	movwf   Tx0
        movlw   LOW(ToneZeroFreq) + 2
	call    GetEE
	movwf   Tx1
        movlw   LOW(ToneZeroFreq) + 1
	call    GetEE
	movwf   Tx2
        movlw   LOW(ToneZeroFreq)
	call    GetEE
	movwf   Tx3

        movlw   LOW(JT9Increment) + 3	;Stored value is 65536 * actual
        call    GetEE
        movwf   B0
        movlw   LOW(JT9Increment) + 2
        call    GetEE
        movwf   B1
        movlw   LOW(JT9Increment) + 1
        call    GetEE
        movwf   B2
        movlw   LOW(JT9Increment)
        call    GetEE
        movwf   B3

	clrf	A1
	clrf	A2
	clrf	A3
        call    Multiply32      ;Result in Acc0/3
				;Add to Tone 0 base code

	movf	Acc2, W		;Divide by 65536 by ignoring lowest Acc
	addwf	Tx0
	
	movf	Acc3, W
	btfsc	STATUS, C
	incfsz	Acc3, W
	addwf	Tx1

	movf	Acc4, W
	btfsc	STATUS, C
	incfsz	Acc4, W
	addwf	Tx2

	movf	Acc5, W
	btfsc	STATUS, C
	incfsz	Acc5, W
	addwf	Tx3
        movf    Tx0 , W
        call    Write9850
        movf    Tx1 , W
        call    Write9850
        movf    Tx2 , W
        call    Write9850
        movf    Tx3 , W
        call    Write9850

        movlw	PllFlag	 ;Phase = 0
        call    Write9850

        bcf     WClk

	bsf     FQud  		;Update DDS 
	nop                     ; 
	nop                     ; 
	bcf     FQud

	return
;-------------------------------
Write9850
        movwf   Temp
        movlw   8
        movwf   BitCount
Loop9850      
        rrf     Temp
        btfss   STATUS , C
        bcf     FData
        btfsc   STATUS , C
        bsf     FData
        nop
        nop
        bsf     WClk     
        nop
        nop
	bcf     WClk       ;data changes with -ve edge of clock
        decfsz  BitCount         
        goto    Loop9850

        return
;-------------------------
SetUpGPS
	bsf	GPSOUT		;Send on off to A2200 module
	call	Delay1ms
	bcf	GPSOUT
	call	Delay1ms
	bsf	GPSOUT		;
	call	Delay1ms
	bcf	GPSOUT
	return
;-----------------------------------------
SendGPSByte                    ;This routine is not used, but kept in for future use, if GPS needs any setup
	movwf   Temp
 if NMEAPol == 1
    bsf     GPSOUT
 else
    bcf     GPSOUT             ;start bit
 endif
    call    GPSHalfBitDelay        ;
    call    GPSHalfBitDelay
	movlw   8
	movwf   Counter
SendGPSDatLoop
	rrf     Temp                ;bit into C


 if NMEAPol == 1
	btfsc   STATUS , C
    bcf     GPSOUT
	btfss   STATUS , C
    bsf     GPSOUT             ;could have 2us jitter depending on 1/0
 else
	btfsc   STATUS , C
    bsf     GPSOUT
	btfss   STATUS , C
    bcf     GPSOUT             ;could have 2us jitter depending on 1/0
 endif

    call    GPSHalfBitDelay
    call    GPSHalfBitDelay
	decfsz  Counter
    goto    SendGPSDatLoop         ;loop 8 + BitDelay long

 if NMEAPol == 1
    bcf     GPSOUT
 else
    bsf     GPSOUT
 endif
    call    GPSHalfBitDelay    
    call    GPSHalfBitDelay        ;Two stop bits
    call    GPSHalfBitDelay
    call    GPSHalfBitDelay

	return
;-------------------------------------------------
 org 0x7B0			
BitMask
	addwf	PCL
	dt	b'10000000'
	dt	b'01000000'
	dt	b'00100000'
	dt	b'00010000'
	dt	b'00001000'
	dt	b'00000100'
	dt	b'00000010'
	dt	b'00000001'	
;--------------------------------------	

SyncTable		;Sync pattern
	addwf	PCL
	dt	b'11001000', b'01000001', b'00000010', b'00000000', b'10100000'
	dt	b'00000000', b'00110010', b'00010000', b'01000000', b'10000000', b'00101000'        

;--------------------------------
	de	"JT9_AD9850_Loc"

;======================================================================
;Variable Flow  :
	; GPS Reads into Stack (Same RAM as SYMBOLDATA below)
	; Generates Seconds, Minutes, Hours, Day, Month, Year, YearHi (using BCDLo, BCDHi)
	; Was old Hash routine now removed
	; Placed into locations in Src0 - SrcD with other fixed data
	; Compressed to S0 - S8
	; Convolved , into Q0 - Q19  means variables thatresus these can no longer be used
	; Interleaved via TARGETDATA (Ram page 1) to give SYMBOLDATA (RAM Page 1)
	; All Page0 variables associated with generating are now redundant
	;     and can be reused for Synth controll and JT9 sending and CW message
	


    cblock 0x20		
	WHi
        SaveSts	
	Flags, Flags2
	Wreg
	Temp, 	Temp2		;Temp2 Used in debugging routines only
	Counter, Counter2	; "      "      "      "     "
	DelCount, DelCount2
	AutoEEADR
	Even, IntlvPointer, Ireg, Jreg
        CommaCount
        SendChar
        DebugTemp
	Seconds, Minutes	;These need to be preserved between calls
        BCDVi, BCDHi, BCDMe, BCDLo 
	BcdTemp1, BcdTemp2
	S0,S1,S2,S3,S4,S5,S6,S7, S8		;Input Packed data.
	A0, A1, A2, A3				;A and B various local variables and 
	B0, B1, B2, B3				; inputs to maths routines
	Acc0, Acc1, Acc2, Acc3, Acc4, Acc5, Acc6, Acc7

	Src0, Src1, Src2, Src3, Src4		;13 bytes of source data for JT9 coding
	Src5, Src6, Src7, Src8, Src9		; addressed Indirectly via Src0)
	SrcA, SrcB, SrcC	  		; Kept and also written to with variable data

	Q0,Q1,Q2,Q3,Q4,Q5,Q6,Q7,Q8,Q9,QA	;Colvolved O/P PLACE IN 
	QB,QC,QD,QE,QF,Q10,Q11,Q12,Q13,Q14  	; sequence as they are addresed indirectly
	Q15,Q16,Q17,Q18,Q19			;These are reused / named below for Location compression routine



    endc   

;Reallocate Qx registers as they're not used until the Hash has been generated after the data has been read.
;Once these are placed into Src0/C (and debug info sent) , they are released for reuse
;    Provided CW message only uses the Src data or its own EE

    cblock Q0		
	Hours, Day, Month, Year		;These do not need to be preserved after they are used
	LatD, LongD
        Sog1, Sog2, Sog3, Sog4		;May eventually want to change format based on speed
    endc


    cblock Q0		;Allocate another block specifically for synth programming
        IntCountLo,  IntCountHi
        SymbCount       ;Counts 0 to 126 for each JT65 symbol
	JT9Data
	PosData
	FreqCache
	LastFreq	;Used to save reprogramming synth if data hasn't changed from last one
	BitCount
	Tx0, Tx1, Tx2, Tx3
        CWChar, CWMsgCnt, CWCharCnt
        MsgCount
    endc



;==========================================
		end
